function [theta1, theta0, out] = ...
    Tanneal(Lambda, Tvec, n, numruns, waitlimit, tempfactor, eta01T, robust)


% Lambda is a vector, whose i-th element is the value lambda0=lambda1 used in the i-th example.
% Tvec is the vector of corresponding initial values of the temperature.
% n is the sample size; numruns the number of replications of the algorithm 
% waitlimit (= L in the paper) is the number of designs which must be
    % tested, without improvement, before the temperature is multiplied by
    % tempfactor and the algorithm restarts (an new 'stage'), using as initial 
    % design the best one found so far.
% eta01T is a triple, [eta_0, eta_1, eta_true] coded by 'M' for MM, 'E' for exponential, 
    % 'L' for linear.  
% set robust = 0 for tau1=tau0=0
% Minsize set at about line 186

%MM = V*x./(K+x)
%Expo = V*(1-exp(-K*x))

% Example 1 (Figures 1 and 2, then 3); true reponse exponential:
% robust = 0: [theta1, theta0, out] = Tanneal([2], [.0002], 20, 5, 500, .95, ['M', 'E', 'E'], 0)
% robust = 1: [theta1, theta0, out] = Tanneal([1, 5], [.0005, .0001], 20, 5, 500, .95, ['M', 'E', 'E'], 1)

% Example 2 (Figures 4 and 5); true response linear:  
% [theta1, theta0, out] = Tanneal([.02, 5], [.00002,.0005], 20, 5, 500, .95, ['M', 'E', 'L'], 1)



x = linspace(.1,5,50); %Set the design space
barheight = 1;
s = 3;

% For each lambda in Lambda, run the annealing algorithm.  Prepare plots of
% (i) comparative 'True mean (solid), closest eta0 (dots) and closest eta1 (dashes)'
% (ii) Annealing progress - D versus trial, 'improvement probability vs. stage' 
    % and 'best design'.  The improvement prob is the relative frequency of 
    % improved designs found in a stage.        
% (iii) Least favourable means and differences between them   
% Then prepare a plot of all best designs for the various lambdas.

for k = 1:length(Lambda)
    lambda = Lambda(k);
    T = Tvec(k);
    [meany, theta00, theta10, Eta0, Eta1, AllDiv, Cumprob, Worstmu0, Worstmu1, ...
        theta1, theta0, Tau1, Tau0, Checkpos, Bestdesign, BestDiv, Bestnxi] = ...
        run1(x, lambda, T, n, numruns, waitlimit, tempfactor, eta01T, robust);
    
    sigmasqd = [1, .5, .1, .01];
    Etad = Eta1 - Eta0;
    maxEtaddsqd = max(Etad.^2);
    for j=1:length(sigmasqd)
        Power(j) = normcdf(sqrt(2*n*BestDiv/(2*sigmasqd(j)))-norminv(.9));
        maxPower(j) = normcdf(sqrt(2*n*maxEtaddsqd/(2*sigmasqd(j)))-norminv(.9));
    end
 
    figure(s*(k-1)+1); 
    plot(x, meany, 'k', x, Eta0, '.', x, Eta1,'--'); 
 %   title({'True mean (solid), closest eta0 (dots) and closest eta1 (dashes)';'  '})
    figure(s*(k-1)+2);
    
  %  subplot(3,1,1);
  %  Trial = 1:length(AllDiv);
  %  BestTrials = Trial(AllDiv==BestDiv);
  %  plot(AllDiv);
  %  set(gca,'XTick', [min(BestTrials), max(BestTrials)]);
  %  xlabel('D vs. trial; ticks at first and last appearances of best design');
  %  %title({['Annealing progress; lambda = ',num2str(lambda)];'  '})
    
    subplot(2,1,1); %%subplot(3,1,2);
    bar(Cumprob(2:length(Cumprob)));
    xlabel('(a)','fontsize',12)

    subplot(2,1,2); %subplot(3,1,3);
    bar(x, Bestnxi);
    axis([0 5.1 0 n]);
    xlabel('(b)','fontsize',12)

    figure(s*(k-1)+3);    
    
    subplot(1,2,1);
    plot(x, Worstmu0, x, Worstmu1);
    xlabel('(a)','fontsize',12)
	%title({['Least favourable means; lambda = ',num2str(lambda)];'  '})

    subplot(1,2,2);
    plot(x, abs(Worstmu0-Worstmu1));
    xlabel('(b)','fontsize',12)
        
    barheight = max(barheight, max(Bestnxi));
    out(k).Div = BestDiv;
    out(k).Power = Power;
    out(k).maxPower = maxPower;
    out(k).Tau = [Tau1 Tau0];
    out(k).Des = Bestdesign;
    figure(s*length(Lambda) + 1)
    subplot(1,length(Lambda),k)
    bar(x, Bestnxi);
    axis([0 5.1 0 barheight+1]);
%    axis([0 5.1 0 6]);
    if k==1
        xlabel ('(a)');
	elseif k==2
        xlabel ('(b)');
	end
end



%% ============================= subfunction "run1" =============================

% Runs the entire annealing algorithm "numrun" times, for one choice of the
% lambdas.  Sets up those terms (eta0, eta1, E(Y|x) etc.) which are only computed onde.
% Calls the function "anneal" numrun times, to do the annealing.


function [meany, theta00, theta10, Eta0, Eta1, AllDiv, Cumprob, Worstmu0, Worstmu1, ...
    theta1, theta0, Tau1, Tau0, Checkpos, Bestdesign, BestDiv, Bestnxi] = ...
    run1(x, lambda, T, n, numruns, waitlimit, tempfactor, eta01T, robust)

if(eta01T(1) == 'M') 
    eta0 = @mm;
    eta0prime = @mmprime;
elseif(eta01T(1) == 'E')
    eta0 = @expo;
    eta0prime = @expoprime;
end

if(eta01T(2) == 'M') 
    eta1 = @mm;
    eta1prime = @mmprime;
elseif(eta01T(2) == 'E')
    eta1 = @expo;
    eta1prime = @expoprime;
end


N = length(x);
indices = 1:N;

problimit = 1/waitlimit; % Continue while at least this fraction of improved states, 
                  % are found in a stage
lambda1 = lambda;
lambda0 = lambda;


% Set up those terms which are only computed once:
theta10 = [1;1];
theta00 = [1;1];

% Simulate E(Y|x)

if(eta01T(3) == 'M')
    meany = mm(x, theta00)';
elseif(eta01T(3) == 'E')
    meany = expo(x, theta10)';
elseif(eta01T(3) == 'L')
    X = [ones(N,1),x'];
    lincoefs = regress(eta0(x,theta00)',X)
    meany = X*lincoefs;
end

% Compute best fitting theta1, together with Eta1 and V1
options = optimset('MaxFunEvals', 1e4,'MaxIter', 1e4);
p1 = length(theta10);
theta1 = lsqnonlin(@(theta) resid1(theta, eta1, x, meany'), theta10, [], [], options); 
Eta1 = eta1(x, theta1)'; % vector of eta1 values
Eta1prime = eta1prime(x, theta1); % matrix of derivatives of eta1
[Q1,R1] = qr(Eta1prime);
V1 = Q1(:,(p1+1):N);

% Compute best fitting theta0, together with Eta0 and V0
p0 = length(theta00);
theta0 = lsqnonlin(@(theta) resid0(theta, eta0, x, meany'), theta00, [], [], options); 
Eta0 = eta0(x, theta0)'; % vector of eta0 values
Eta0prime = eta0prime(x, theta0);  % matrix of derivatives of eta2
[Q0,R0] = qr(Eta0prime);
V0 = Q0(:,(p0+1):N);

Etad = Eta1 - Eta0;

if robust == 0   
    P = zeros(N);
else
    P = V0*V0'/lambda0 + V1*V1'/lambda1;
end
minsize = 6; %%% max(p0,p1);  % Set the minimum allowable number of design points
BestDiv = 0;

for run = 1:numruns    
    [tau1, tau0, c0, c1, checkpos, bestnxi, bestDiv, allDiv, cumprob] = ...
    anneal(n, N, indices, T, waitlimit, tempfactor, problimit, ...
    Etad, V1, V0, lambda1, lambda0, p1, p0, P, minsize, robust);
    run
    checkpos
    bestDiv
    if checkpos > 0 & bestDiv > BestDiv
        Bestnxi = bestnxi;
        BestDiv = bestDiv;
        Worstc0 = c0;
        Worstc1 = c1;
        AllDiv = allDiv;
        Cumprob = cumprob;
        Tau1 = tau1;
        Tau0 = tau0;
        Checkpos = checkpos;
    end
end

Bestdesign = [x(Bestnxi>0); Bestnxi(Bestnxi>0)];
Worstmu0 = Eta0 + V0*Worstc0;
Worstmu1 = Eta1 + V1*Worstc1;

	

%% ============================= subfunction "anneal" =============================

function [tau1, tau0, c0, c1, checkpos, bestnxi, bestDiv, allDiv, cumprob] = ...
    anneal(n, N, indices, T, waitlimit, tempfactor, problimit, ...
    Etad, V1, V0, lambda1, lambda0, p1, p0, P, minsize, robust)
     
% Randomly choose an initial design of size n from 1:N
design = ceil(N.*rand(1,n));
nxi = zeros(1,N);
for j = 1:n
    k = design(j);
    nxi(k) = sum(design==k);
end

Div = ncp(nxi, P, Etad, n); % Compute Div of selected design
bestnxi = nxi;  % Initialize bestnxi, bestDiv and allDiv
bestDiv = Div;
allDiv = Div;
cumprob = 1;

while cumprob(length(cumprob)) >= problimit 
    nxi = bestnxi; % Restart, using the best design found so far
    Div = bestDiv;    
    wait = 0;
    num_better = 0;
    num_worse = 0;
    while wait < waitlimit
        %% Update the design:
        oldnxi = nxi;
        oldDiv = Div;
        % Choose kklimit in 1:minsize of the current sites, at random;
        % subtract one from each of them
        kklimit = ceil(minsize.*rand(1));  % kklimit sites will be chosen
        pos = indices(nxi>0); % these are sites with positive frequencies
        s = length(pos);
        revisedkklimit = kklimit;
        for kk = 1:kklimit
            choice = ceil(s.*rand(1));
            location1 = pos(choice); % choose one of them
            if nxi(location1) > 0
                nxi(location1) = nxi(location1) - 1; % if freq is still > 0, 
                                                    % reduce it by one
            else
                revisedkklimit = revisedkklimit-1;  % otherwise reduce the value of 
                                       % kklimit to be used next
            end
        end        

        % Choose kklimit of 1:N at random, add one to each of them
        for kk = 1:revisedkklimit
            location2 = ceil(N.*rand);
            nxi(location2) = nxi(location2) + 1;
        end
        
        pos = indices(nxi>0);
        s = length(pos);

        % Decide whether or not to accept the new state:
        if s < minsize  % reject if too few support points
            nxi = oldnxi;
            Div = oldDiv;
        else
            Div = ncp(nxi, P, Etad, n); 
            if Div > oldDiv
                num_better = num_better + 1;
            else
                num_worse = num_worse + 1;
                acceptance_prob = exp(-(oldDiv-Div)/T);
                if rand > acceptance_prob  % if so, reject the new xi
                    nxi = oldnxi;
                    Div = oldDiv;
                end              
            end             
        end
        allDiv = [allDiv; Div];
        if Div > bestDiv % update the bestDiv
            bestDiv = Div;
            bestnxi = nxi;
            wait = 0;
        else
            wait = wait + 1; % increment wait if Div has not increased
        end
    end
cumprob = [cumprob; num_better/(num_better + num_worse)]; % update cumprob
T = tempfactor*T;  % reduce the temperature
end

% Determine c1 and c0; compute and check the taus:
indices = 1:N;
pos = indices(nxi>0);
s = length(pos);
Pplus = P(pos,pos);
rootDplus = diag(sqrt(nxi(pos)))/sqrt(n);
inv = (eye(s) + rootDplus*Pplus*rootDplus)\eye(s);
rootDplus = diag(sqrt(nxi(pos)))/sqrt(n);
if robust == 0 
    c = zeros(2*N-p1-p0, 1);
else
    c = [V0(pos,:)'/lambda0;V1(pos,:)'/lambda1]...
    *rootDplus*inv*rootDplus*Etad(pos);
end
c0 = c(1:(N-p1));
c1 = -c((N-p1+1):(2*N-p1-p0));
tau1 = norm(c1);
tau0 = norm(c0);
checkpos = norm(Etad) - tau1 - tau0;
    

%% ============================= subfunction "Div" =============================
function Div = ncp(nxi, P, Etad, n) 
indices = 1:length(nxi);
pos = indices(nxi>0);
s = length(pos);
Pplus = P(pos,pos);
rootDplus = diag(sqrt(nxi(pos)))/sqrt(n);
inv = (eye(s) + rootDplus*Pplus*rootDplus)\eye(s);
Div = norm(inv*rootDplus*Etad(pos))^2;

%% ============================= subfunction "mm" =============================
function mm = mm(x, theta) 
V = theta(1);
K = theta(2);
mm = V*x./(K+x);

%% ============================= subfunction "expo" =============================
function expo = expo(x, theta) 
V = theta(1);
K = theta(2);
expo = V*(1-exp(-K*x)); 


%% ============================= subfunction "mmprime" =============================
function mmprime = mmprime(x, theta) 
V = theta(1);
K = theta(2);
mmprime = [x./(K+x); -V*x./((K+x).^2)]';

%% ============================= subfunction "expoprime" =============================
function expoprime = expoprime(x, theta) 
V = theta(1);
K = theta(2);
expoprime = [1-exp(-K*x); V*x.*exp(-K*x)]'; 

%% ============================= subfunction "resid1" =============================
function resid1 = resid1(theta, eta1, x, meany) 
resid1 = meany - eta1(x, theta);

%% ============================= subfunction "resid0" =============================
function resid0 = resid0(theta, eta0, x, meany) 
resid0 = meany - eta0(x, theta);



